/*
Copyright (C) 2000 Mathieu Decore <mdecor@ens.uvsq.fr>
                   Marc Tanguy <mtanguy@ens.uvsq.fr>

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le 
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence, ou
 encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE 
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER. 
Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
 temps que ce programme ; 
si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

#include "lecteur.h"

int
initialisation (int argc, char *argv[])
{
  pager = getenv ("PAGER");
  if (!pager)
  {
    pager = pager_defaut;
  }
  home = getenv ("HOME");
  if (!home)
  {
    lecteur_error (NULL, "Pas de variable d'environnement HOME\n");
  }
  if ((strlen (home)) > TAILLE_CHEMIN_MAX)
  {
    lecteur_error (NULL,
		   "Variable HOME suprieure  %d caractres\n",
		   TAILLE_CHEMIN_MAX);
  }
  if (argc == 3)
  {
    if (strlen (argv[2]) < TAILLE_CHEMIN_MAX)
    {
      CheminForumServeur = argv[2];
    }
    else
    {
      lecteur_error (NULL, "Rpertoire `%s' trop long\n", argv[2]);
    }
  }
  else
  {
    CheminForumServeur = getenv ("NEWSSERVER");
    if (CheminForumServeur == NULL)
    {
#ifdef CHEMIN_SERVEUR
			CheminForumServeur = CHEMIN_SERVEUR;
#else 
      lecteur_error (NULL, "Pas de variable d'environnement NEWSSERVER");
#endif /* !CHEMIN_SERVEUR */
   }
    if ((strlen (CheminForumServeur)) > TAILLE_CHEMIN_SERVEUR_MAX)
    {
      lecteur_error (NULL,
		     "Variable NEWSSERVER suprieure  %d caractres\n",
		     TAILLE_CHEMIN_SERVEUR_MAX);
    }
  }

  /*
   * dfinition du Chemin des Articles du Serveur
   */
  memset (CheminArticles, 0, TAILLE_CHEMIN_MAX);
  strncpy (CheminArticles, CheminForumServeur, TAILLE_CHEMIN_SERVEUR_MAX);
  strncat (CheminArticles, "/Articles", TAILLE_CHEMIN_MAX);

  /*
   * dfinition du Chemin du ~/.forum
   */
  memset (NomFichierForumClient, 0, TAILLE_CHEMIN_MAX);
  strncpy (NomFichierForumClient, home, TAILLE_CHEMIN_MAX);
  strncat (NomFichierForumClient, "/.forum",
	   TAILLE_CHEMIN_NOM_MAX - strlen (NomFichierForumClient) + 1);
  return 0;
}

void
RechercheNombreArticlesServeur (void)
{
  FILE           *FichierForumServeur;
  int             retour = 0;
  char            NomFichierForumServeur[TAILLE_CHEMIN_NOM_MAX];

  /*
   * Initialisation
   */
  memset (NomFichierForumServeur, 0, TAILLE_CHEMIN_NOM_MAX);

  /*
   * Recherche du chemin/nom du fichier FORUM du Serveur
   */
  strncpy (NomFichierForumServeur, CheminForumServeur, TAILLE_CHEMIN_MAX);
  strncat (NomFichierForumServeur, "/.forumSRV",
	   TAILLE_CHEMIN_NOM_MAX - strlen (NomFichierForumServeur) - 1);

  /*
   * Lecture du Nombre d'articles, Numero du premier article et Numero
   * du dernier article
   */
  FichierForumServeur = fopen (NomFichierForumServeur, "r");
  if (FichierForumServeur == NULL)
  {
    lecteur_error ("fopen ",
		   "Erreur lors de l'ouverture du fichier %s\n",
		   NomFichierForumServeur);
  }
  retour = fscanf (FichierForumServeur, "%lu%lu%lu", &NombreArticles,
		   &NumeroPremierArticle, &NumeroDernierArticle);
  if (retour != 3)
  {
    lecteur_error ("fscanf ",
		   "Erreur pendant la lecture de $SERVEUR/.forumSRV\n");
  }

  /*
   * Verification de la coherence du fichier .forumSRV
   */
  if (NombreArticles == 0)
  {
    lecteur_error (NULL,
		   "Pas d'article  lire sur le serveur dans : %s\n",
		   NomFichierForumServeur);
  }
  if ((NumeroDernierArticle - NumeroPremierArticle + 1) != NombreArticles)
  {
    lecteur_error (NULL, "Erreur de cohrence de %s\n",
		   NomFichierForumServeur);
  }

  /*
   * Fermeture du $NEWSSERVER/.forumSRV
   */
  fclose (FichierForumServeur);
}

void
ListeArticlesNonLus (void)
{
  FILE           *FichierForumClient;
  int             retour = 0;
  NbArticles_t    NumeroArticle = 0;

  /*
   * Lecture du $HOME/.forum o sont stockes les numeros des articles
   * deja lus S'il n'existe pas alors il n'y a pas d'articles lus
   */
  FichierForumClient = fopen (NomFichierForumClient, "r");
  if (FichierForumClient == NULL)
  {	
	  errno = 0;
    return;
  }

  /*
   * Pour chaque article existant on ecrit un 1 dans un tableau de
   * taille 'NombreArticles'
   */
  while (1)
  {
    retour = fscanf (FichierForumClient, "%lu", &NumeroArticle);
    if (feof (FichierForumClient))
    {
      if (retour == EOF)
      {
	break;
      }
      else
      {
	lecteur_error ("fscanf ", "Erreur  la fin de fichier ~/.forum\n");
      }
    }
    if (retour != 1)
    {
      lecteur_error ("fscanf",
		     "Erreur  la lecture interne du fichier ~/.forum\n");
    }
    if (NumeroArticle < (NombreArticles + NumeroPremierArticle))
    {
      TableauForum[NumeroArticle - NumeroPremierArticle] = 1;
    }
  }

  /*
   * Fermeture du ~/.forum
   */
  fclose (FichierForumClient);
}

void
AffichageListeArticlesNonLus (void)
{
  FILE           *FichierTemporaire;
  FILE           *FichierTemporaireArticle;

  char            NomFichierTemporaire[L_tmpnam + 1];
  char            Auteur[TAILLE_BUF];
  char            Sujet[TAILLE_BUF];

  char            tampon[TAILLE_BUF];
  char            NomArticle[TAILLE_CHEMIN_NOM_MAX];
  char            NomArticleTemporaire[TAILLE_NOM_ARTICLE];
  char           *ptrRetour;
  char           *retour;
  int             val;
  pid_t           pid;
  NbArticles_t    i = 0;

  /*
   * Initialisation
   */
  memset (NomFichierTemporaire, 0, L_tmpnam + 1);
  retour = tmpnam (NomFichierTemporaire);
  if (retour == NULL)
  {
    lecteur_error ("tmpnam ",
		   "Impossible de crer un nom de fichier temporaire");
  }

  /*
   * Ouverture et Remplissage de la Liste Temporaire d'article non lus
   */
  FichierTemporaire = fopen (NomFichierTemporaire, "w+");
  if (!FichierTemporaire)
  {
    lecteur_error ("fopen ",
		   "Erreur  l'ouverture en criture du fichier %s",
		   NomFichierTemporaire);
  }
  fprintf (FichierTemporaire, "----------------------------------------");
  fprintf (FichierTemporaire, "----------------------------------------\n");
  fprintf (FichierTemporaire, "|  Numero   |       Auteur            | ");
  fprintf (FichierTemporaire, "          Sujet                        |\n");
  fprintf (FichierTemporaire, "----------------------------------------");
  fprintf (FichierTemporaire, "----------------------------------------\n");
  for (i = 0; i < NombreArticles; i++)
  {
    if (TableauForum[i] == 0)
    {				/* si l'article i est non lu alors afficher */

      /*
       * Lecture/Ecriture du numero de l'article
       */
      fprintf (FichierTemporaire, "| %9lu", NumeroPremierArticle + i);
      sprintf (NomArticleTemporaire, "%lu", NumeroPremierArticle + i);

      /*
       * Ouverture de l'article pour extraction
       * Auteur/Sujet
       */
      strcpy (NomArticle, CheminArticles);
      strcat (NomArticle, "/");
      strncat (NomArticle, NomArticleTemporaire,
	       TAILLE_CHEMIN_NOM_MAX - strlen (NomArticle) - 1);

      FichierTemporaireArticle = fopen (NomArticle, "r");
      if (!FichierTemporaireArticle)
      {
			  lecteur_error ("fopen ",
		                   "Erreur  l'ouverture en lecture du fichier %s",
		                   NomArticle);
      }

      /*
       * Lecture/Ecriture de l'auteur
       */
      memset (Auteur, 0, TAILLE_BUF);

      ptrRetour = fgets (Auteur, TAILLE_BUF, FichierTemporaireArticle);
      if (feof (FichierTemporaireArticle))
      {
	if (ptrRetour == NULL)
	{
	  lecteur_error ("fgets ", "Article %s vide", NomArticle);
	}
	else
	{
	  lecteur_error ("fgets ",
			 "Erreur dans la lecture de %s", NomArticle);
	}
      }
      else if (ptrRetour == NULL)
      {
	lecteur_error ("fgets ", "Erreur dans la lecture de %s", NomArticle);
      }
      strtok (Auteur, ":");
      strncpy (Auteur, strtok (NULL, ""), TAILLE_BUF);
      Auteur[strlen (Auteur) - 1] = '\0';
      Auteur[23] = '\0';
      fprintf (FichierTemporaire, " | %23s", Auteur);

      /*
       * On ne recupere pas la date
       */
/*                      memset(tampon, 0, TAILLE_BUF); */
      ptrRetour = fgets (tampon, TAILLE_BUF, FichierTemporaireArticle);

      /*
       * Lecture/Ecriture du sujet
       */
      memset (Sujet, 0, TAILLE_BUF);
      ptrRetour = fgets (Sujet, TAILLE_BUF, FichierTemporaireArticle);
      if (ptrRetour == NULL)
      {
	    lecteur_error ("fgets ", "Erreur dans la lecture de %s", NomArticle);
      }
      strtok (Sujet, ":");
      strncpy (Sujet, strtok (NULL, ""), TAILLE_BUF);
      Sujet[strlen (Sujet) - 1] = '\0';
      Sujet[38] = '\0';
      fprintf (FichierTemporaire, " | %38s |\n", Sujet);
      fclose (FichierTemporaireArticle);
      val = fflush (FichierTemporaire);
      if (val)
      {
			lecteur_error ("fflush ",
		       "Erreur d'criture des donnes du buffer\n");
      }
    }
  }
  fprintf (FichierTemporaire, "----------------------------------------");
  fprintf (FichierTemporaire, "----------------------------------------\n");
  fflush (FichierTemporaire);

  pid = fork ();
  switch (pid)
  {
  case -1:
    lecteur_error ("fork ", "Erreur dans la creation d'un processus fils");

  case 0:
    if (execlp (pager, pager, NomFichierTemporaire, NULL) == -1)
    {
      lecteur_error ("execlp ", "Erreur lors de execlp\n");
    }
  default:
    wait (NULL);
    break;
  }

  /*
   * Fermeture/Destruction du fichier temporaire
   */
  fclose (FichierTemporaire);
  unlink (NomFichierTemporaire);
}

int
AfficheArticle (NbArticles_t NumeroArticle)
{
  char            NomArticle[TAILLE_NOM_ARTICLE];
  char            CheminArticle[TAILLE_CHEMIN_MAX];
  pid_t           pid;

  if ((NumeroArticle < NumeroPremierArticle) ||
      (NumeroArticle > NumeroDernierArticle))
  {
    return 1;			/* le printf est assur par le marquage du fichier comme lu */
  }
  strncpy (CheminArticle, CheminArticles, TAILLE_CHEMIN_MAX);
  sprintf (NomArticle, "/%lu", NumeroArticle);
  strncat (CheminArticle, NomArticle, TAILLE_NOM_ARTICLE);

  pid = fork ();
  switch (pid)
  {
  case -1:
    lecteur_error ("fork ", "Erreur dans la creation d'un processus fils\n");

  case 0:
    if (execlp (pager, pager, CheminArticle, NULL) == -1)
    {
      lecteur_error ("execlp ", "Erreur lors de execlp\n");
    }
  default:
    wait (NULL);
    break;
  }
  return 0;
}

int
MarqueArticle (NbArticles_t NumeroArticle)
{
  if ((NumeroArticle < NumeroPremierArticle) ||
      (NumeroArticle > NumeroDernierArticle))
  {
    printf
      ("Vous avez entre un numero d'article non compris entre %lu et %lu\n",
       NumeroPremierArticle, NumeroDernierArticle);
    return 1;
  }
  TableauForum[NumeroArticle - NumeroPremierArticle] = 1;
  return 0;
}

void
MarqueTousArticles (void)
{
  NbArticles_t    i;

  for (i = 0; i < NombreArticles; i++)
  {
    TableauForum[i] = 1;
  }
}

void
EcritFichierForumClient (void)
{
  NbArticles_t    i;
  FILE           *FichierForumClient;

  FichierForumClient = fopen (NomFichierForumClient, "w");
  if (FichierForumClient == NULL)
  {
    lecteur_error ("fopen ",
		   "Erreur : ouverture en ecriture du fichier $HOME/.forum\n");
  }
  for (i = 0; i < NombreArticles; i++)
  {
    if (TableauForum[i] == 1)
    {
      fprintf (FichierForumClient, "%lu\n", NumeroPremierArticle + i);
    }
  }
  fclose (FichierForumClient);
}

int
menu (void)
{
  unsigned char  *choix;
  int             retour;
  char            lettre;
  NbArticles_t    nombre;

  choix = (unsigned char *) malloc ((TAILLE_NOM_ARTICLE + 2) * sizeof (char));

  memset (choix, 0, TAILLE_NOM_ARTICLE + 1);
  lettre = 'q';
  nombre = 0;

  printf ("\033[2J");

  while (1)
  {
    printf ("Menu\n");
    printf (" q = quitte le lecteur \n");
    printf (" l = liste des articles non lus \n");
    printf (" d + Nombre = marque l'article comme lu \n");
    printf (" D = marque tous les articles comme lus \n");
    printf (" Nombre = numero d'article a lire \n");
    printf ("\n");
    printf ("Votre choix ? ");
    fgets ((char *) choix, TAILLE_NOM_ARTICLE, stdin);
    choix[TAILLE_NOM_ARTICLE + 1] = '\0';

    if (isalpha (choix[0]))
    {
      retour = sscanf ((char *) choix, "%c%lu", &lettre, &nombre);
      if ((retour != 2) && (retour != 1))
      {
	      printf ("Mauvais choix\n");
      }
      switch (lettre)
      {

	/*
	 * Quit definitivement le Lecteur
	 */

      case 'q':		/* Ecriture du fichier ~/.forum
				   * contenant la liste des articles
				   * lus */
	EcritFichierForumClient ();
	free (choix);
	free (TableauForum);
	return 0;

	/*
	 * Affichage formate de la liste des articles
	 * non lus avec $PAGER
	 */

      case 'l':
	AffichageListeArticlesNonLus ();
	break;

	/*
	 * Marque tous les articles du serveur comme
	 * lus
	 */

      case 'D':
	MarqueTousArticles ();
	break;

	/*
	 * Marque l'article du serveur 'Numero' comme
	 * lu
	 */

      case 'd':
	MarqueArticle (nombre);
	break;
      }
    }
    else
    {
      /*
       * Lecture du message 'Numero'
       */
      retour = sscanf ((char *) choix, "%lu", &nombre);
      fflush (NULL);
      if (retour != 1)
      {
	printf ("Mauvais choix\n");
      }
      else
      {
	AfficheArticle (nombre);
	MarqueArticle (nombre);
      }
    }
  }
}

int
main (int argc, char *argv[])
{
  /*
   * Initialisation des variables d'environnement
   */
 switch (argc)
  {
  case 1:
    initialisation (argc, argv);
		break;
 	case 3:
    if ((argv[1][0] == '-') && (argv[1][1] == 'd'))
    {
      initialisation (argc, argv);
			break;
    }
  default:
    if (!strcmp (argv[1], "--version"))
    {
      fprintf (stdout, "Lecteur de News 1.0\n");
      exit (0);
    }
    if (!strcmp (argv[1], "--help"))
    {
      printf ("Usage : lecteur [-d repertoire des articles du serveur]\n");
      printf ("Lire les articles dans le repertoire %s\n", CheminArticles);
      printf ("\n  --help        afficher l'aide-mmoire\n");
      printf ("  --version     afficher le nom et la version du logiciel\n");
      printf ("\nRapporter toutes anomalies a <mtanguy@ens.uvsq.fr> ");
      printf ("ou <mdecor@ens.uvsq.fr>\n");
      exit (0);
    }
    printf ("lecteur : Les paramtres ne sont pas corrects.\n");
    printf ("Pour en savoir davantage, faites: `lecteur --help'.\n");
    exit (0);
  }

  /*
   * Recherche du nombre d'articles contenus sur le serveur grace au
   * fichier $SERVEUR/.forum
   */
  RechercheNombreArticlesServeur ();

  TableauForum = (char *) malloc (NombreArticles * sizeof (char));

  if (!TableauForum)
  {
    lecteur_error ("malloc ", "Allocation mmoire\n");
  }
  memset (TableauForum, 0, NombreArticles);

  /*
   * Comparaison avec le nombre d'articles deja lus et modification du
   * TableauForum[] en consequence
   */
  ListeArticlesNonLus ();

  /*
   * Menu
   */
  menu ();
  exit (0);
}
