/*
Copyright (C) 2000 Mathieu Decore <mdecor@ens.uvsq.fr>
                   Marc Tanguy <mtanguy@ens.uvsq.fr>

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le 
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence, ou
 encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE 
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER. 
Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
 temps que ce programme ; 
si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <pwd.h>
#include <limits.h>
#include <sys/stat.h>
#include "posteur.h"

int my_setreuid (int ruid, int euid)
{
#ifdef __hpux
  return setreuid (ruid, euid);
#else
  return seteuid (euid);
#endif
}

void Lire_Sujet (char *sujet)
{

  char buf[TAILLE_BUF];

  printf ("Subject : ");
  fgets (buf, TAILLE_BUF, stdin);
  sprintf (sujet, "Subject : %s\n", buf);

}

void Creer_Fichier_Corps_Message (char *Nom_Temp_Article)
{
  int Temp_FD;

  Temp_FD = mkstemp (Nom_Temp_Article);

  if (Temp_FD == -1)
    { 
      fprintf (stderr, "%s:%d : Erreur lors de mkstemp %s\n", 
	       __FILE__, __LINE__, Nom_Temp_Article);
      perror ("mkstemp ");
      exit (errno);
    }
  
  chmod (Nom_Temp_Article, 0644);
}

char * Trouver_Editeur ()
{
  
  char *editeur;
  
  /*
   * Recuperation du contenu de la variable d'environnement EDITOR definie
   * par l'utilisateur, contenant le nom d'un editeur de fichiers
   */

  editeur = getenv ("EDITOR");
  
  /*
   * Si la variable d'environnement EDITOR n'est pas definie, vi sera utilise
   * pour saisir le contenu de l'article
   */

  if (editeur == NULL)
    {
      editeur = "vi"; 
    }
  
  return (editeur);
}

void Editer_Article (char *editeur, char *Nom_Corps_Article)
{
  pid_t pid;

/*  printf ("Edition : %d %d\n", getuid (), geteuid ()); */

  pid = fork() ;
  if (pid == 0)
    {
      if (execlp (editeur, editeur, Nom_Corps_Article, NULL) == -1)
	{
	  fprintf (stderr, "%s:%d : Erreur lors de execlp\n", 
		   __FILE__, __LINE__);
	  exit (5);
	}
    }
  wait (NULL);
  printf ("Nom_Corps_Article : %s\n", Nom_Corps_Article);
}

void Sauvegarder_Article_Mort (char *Home, char *Nom_Corps_Article)
{
  char Nom_Mort[TAILLE_NOM + TAILLE_CHEMIN];
  int Mort_FD;

  strcpy (Nom_Mort, Home);
  strcat (Nom_Mort, "/dead.news");  

  Mort_FD = open (Nom_Mort, O_CREAT | O_TRUNC | O_WRONLY, 0644);
  
  if (rename (Nom_Corps_Article, Nom_Mort) == -1)
    {
      fprintf (stderr, "%s:%d : Erreur lors de renommage du fichier\n "
	       "%s en\n %s.\n", 
	       __FILE__, __LINE__, Nom_Corps_Article, Nom_Mort);
      
      perror ("rename ");
      close (Mort_FD);
      exit (errno);
    }
  
  fprintf (stderr, "Fichier sauvegarde : %s.\n", Nom_Mort);

  close (Mort_FD);
  
  exit (103);
}

void Verifier_Poster (char *Home, char *Nom_Corps_Article)
{

  char c[2];

  printf ("Poster cet article (o/n [o]) ?\n");

  fgets (c, 2, stdin);
  printf ("%s\n", c);
  
  switch (c[0])
    {
      case 'o' : 
	{
	  break;
	}
	
      case 'n' : 
	{
	  Sauvegarder_Article_Mort (Home, Nom_Corps_Article);
	  break;
	}
       
      case '\n' :
	{
	  break;
	}
	
      default :
	{
	  Verifier_Poster (Home, Nom_Corps_Article);
	}
    }
}

void Verifier_Taille_Article (char *Home, char *Nom_Corps_Article)
{
  struct stat stbuf;
  
  /* 
   * Consultation du descripteur du fichier contenant le coprs du message
   */

  if (stat (Nom_Corps_Article, &stbuf) != 0)
    {
      perror ("stat ");
      exit (errno);
    }
  
  /* 
   * Si le corps du message depasse 1 Mo, il est sauve dans le fichier
   * dead.news. On quitte le programme.
   */

  if (stbuf.st_size > TAILLE_MAX_FICHIER_ARTICLE)
    {
      fprintf (stderr, "\nMessage trop long.\n\n");
    }

}

int Creer_Article_Complet (char *Chemin_Articles, char *Nom_Article)
{
  int Article_FD ;  
  
  /*
   * Creation du nom complet de l'article : d'abord le chemin de l'article,
   * definit par CHEMIN_ARTICLE passe en argument a la compilation (voir
   * le Makefile), ensuite concatenation du nom du fichier
   */

  strcpy (Nom_Article, Chemin_Articles);
  strcat (Nom_Article, "article.XXXXXX");
  
  /*
   * Creation du fichier contenant l'article. Le nom de ce fichier est
   * unique grace a l'utilisation de mkstemp
   */


/*  printf ("ruid=%d euid=%d\n", getuid (), geteuid ()); */

  Article_FD = mkstemp (Nom_Article);

  if (Article_FD == -1)
    { 
      fprintf (stderr, "%s:%d : Erreur lors de mkstemp %s\n", 
	       __FILE__, __LINE__, Nom_Article);      
      perror ("mkstemp ");
      exit (errno);
    }  
  
  chmod (Nom_Article, 0644);
  
  return (Article_FD);
}

void Ecrire_Nom_Utilisateur (uid_t uid, int Article_FD)
{
  struct passwd *user;
  char buf[TAILLE_BUF];

  /*
   * Recuperation des champs correspondant a l'utilisateur dans le fichier
   * /etc/passwd
   */

  user = getpwuid (uid);
  
  if (user == NULL)
    {
      perror ("getpwuid ");
      exit (errno);
    }

  /*
   * Ecriture du login de l'utilisateur et de son vrai nom
   */

  sprintf (buf, "From : %s (%s)\n", user->pw_name,
			  user->pw_gecos);
  write (Article_FD, buf, strlen (buf));
}

void Ecrire_Date_Postage (int Article_FD)
{
  time_t t;
  char *Date;
  char buf[TAILLE_BUF];

  /*
   * Recuperation de la date courante en secondes depuis le 
   * 01/01/1970 00.00 GMT
   */

  time (&t);

  /*
   * Conversion du nombre de secondes en date lisible
   */

  Date = ctime (&t);

  /*
   * Ecriture dans le fichier temporaire de la date mise en forme
   */
 
  sprintf (buf, "Date : %s", Date);
  write (Article_FD, buf, strlen (buf));

}

void Ecrire_Corps_Article (char *Nom_Corps_Article, int Article_FD)
{

  FILE *Corps_Article;

  char buf[TAILLE_BUF];
  char Fin_Article = '\n';
  int nlus;

  Corps_Article = fopen (Nom_Corps_Article, "r");

  if (Corps_Article == NULL)
    {
      fprintf (stderr,
	       "%s:%d : Erreur lors de l'ouverture en lecture du fichier "
	       "%s.\n",
	       __FILE__, __LINE__, Nom_Corps_Article);
      perror ("fopen ");
      exit (errno);
    }  
  
  while ((nlus = fread (buf, sizeof (char), TAILLE_BUF, Corps_Article)) > 0)
    {
      write (Article_FD, buf, nlus);
    }

  /*
   * On rajoute un saut de ligne en fin de fichier, au cas ou le corps du
   * message se terminerait par une ligne non vide
   */
  
  write (Article_FD, &Fin_Article, sizeof (char));
  
  fclose (Corps_Article);
}

void Editer_Article_Complet (uid_t uid, char *sujet, char *Nom_Corps_Article,
			     int Article_FD)
{
  
  /*
   * Ecriture du login et du nom de l'utilisateur posteur de l'article
   */

  Ecrire_Nom_Utilisateur (uid, Article_FD);

  /*
   * Ecriture de la date de postage de l'article
   */

  Ecrire_Date_Postage (Article_FD);

  /*
   * Ecriture du sujet de l'article
   */

  write (Article_FD, sujet, strlen (sujet));

  /* 
   * Ecriture du message a partir du fichier temporaire contenant le corps
   * de l'article a poster
   */

  Ecrire_Corps_Article (Nom_Corps_Article, Article_FD);  
}


void Envoyer_Message_Serveur (char *Chemin_Serveur, char *Nom_Article)
{
  char Nom_Tube[TAILLE_NOM + TAILLE_CHEMIN];
  int Tube_FD;

  strcpy (Nom_Tube, Chemin_Serveur);
  strcat (Nom_Tube, "/newspipe");  

  Tube_FD = open (Nom_Tube, O_WRONLY | O_NDELAY);

  if (Tube_FD == -1)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'ouverture de %s\n.", 
	       __FILE__, __LINE__, Nom_Tube);
      perror ("open ");
      exit (errno);
    }

  write (Tube_FD, Nom_Article, strlen (Nom_Article));

  close (Tube_FD);
}

int main (int argc, char *argv[])
{
  char Nom_Article[TAILLE_NOM + TAILLE_CHEMIN];
  char *Nom_Corps_Article;
  char sujet[TAILLE_BUF];

  char Chemin_Articles[TAILLE_CHEMIN];
  char Chemin_Serveur[TAILLE_CHEMIN];
  
  int Article_FD;
  char *editeur;

  struct passwd *user;
  uid_t uid;  

  int ruid, euid;

  char *Login;
  char *Home;

  int i;
  
  struct tm;

  for (i=0; i<argc; i++)
    {
      if (!strcmp (argv[i], "-h"))
	{
	  printf ("Usage : %s [-r num] [-p chemin]\n", argv[0]);
	  exit (0);
	}
    }
  
  strcpy (Chemin_Serveur, CHEMIN_SERVEUR);
  
  strcpy (Chemin_Articles, Chemin_Serveur);
  strcat (Chemin_Articles, "/Articles/");

  ruid = getuid ();
  euid = geteuid ();

  /* 
   * Determination du login de l'utilisateur du posteur
   */

  uid = getuid();
  user = getpwuid(uid);

  Login = user->pw_name;

  /*
   * Determination du repertoire d'accueil de l'utilisateur. C'est dans ce
   * repertoire que sera stocke le fichier temporaire contenant le corps du
   * message, ainsi que le fichier dead.news si une erreur s'est produite
   */

  Home = user->pw_dir;

  /*
   * Creation du nom complet du fichier temporaire contenant le corps du
   * message : d'abord le repertoire d'accueil de l'utilisateur, ensuite
   * concatenation du nom du fichier
   */

  /*  strcat (Nom_Corps_Article, Home); */
  Nom_Corps_Article = (char *) malloc ((TAILLE_NOM + TAILLE_CHEMIN) * 
				       sizeof (char));
  
  strcpy (Nom_Corps_Article, Home);
  strcat (Nom_Corps_Article, "/news_tmp.XXXXXX");
  

  /*
   * Saisie du sujet du message
   */

  Lire_Sujet (sujet);

  /*
   * Le processus courant (le posteur) se voit attribuer comme identite 
   * reelle celle de l'utilisateur effectif (c'est a dire l'utilisateur qui 
   * a invoque le programme) et comme identite effective celle de 
   * l'utilisateur reel (c'est a dire l'utilisateur qui est proprietaire du
   * du posteur). Cela permet a l'utilisateur qui a invoque le programme
   * d'etre proprietaire du fichier contenant l'article, et l'empeche au cours
   * de l'edition du message de pouvoir editer des fichier appartenant au 
   * proprietaire du posteur
   */

/*   printf ("Tout debut : %d %d\n", getuid (), geteuid ()); */

  my_setreuid (ruid, ruid);
/*   printf ("Au debut : %d %d\n", getuid (), geteuid ()); */

  /*
   * Creation et ouverture du fichier temporaire contenant le corps du message
   */

  Creer_Fichier_Corps_Message (Nom_Corps_Article);

  /*
   * Determination de l'editeur a utiliser
   */

  editeur = Trouver_Editeur ();

  /*
   * Edition du corps du message
   */

  Editer_Article (editeur, Nom_Corps_Article);

  Verifier_Poster (Home, Nom_Corps_Article);

  /*
   * Verification de la taille de l'article : les articles de taille 
   * superieure a TAILLE_MAX_FICHIER_ARTICLE ne sont pas postes
   */

  Verifier_Taille_Article (Home, Nom_Corps_Article);

  /*
   * Le proprietaire du posteur (egalement proprietaire du serveur)
   * devient proprietaire des fichiers generes par le posteur. Cela permet
   * au serveur de renommer ce fichier par la suite
   */

/*   printf ("Avant creation %d %d\n", getuid(), geteuid()); */

  if (my_setreuid (ruid, euid) == -1)
    {
      perror ("my_setreuid ");
      exit (errno);
    }
  

  printf ("ruid=%d euid=%d\n", ruid, euid);
  
/*   printf ("Avant creation2 %d %d\n", getuid(), geteuid()); */
  
  /* 
   * Creation du fichier temporaire unique contenant le message complet
   * et attribution des bons droits
   */

  Article_FD = Creer_Article_Complet (Chemin_Articles, Nom_Article);

  /*
   * Mise en forme de l'en tete du message 
   */

  Editer_Article_Complet (ruid, sujet, Nom_Corps_Article,
			  Article_FD);
  close (Article_FD);

 
  /* 
   * Ouverture du tube nomme et envoie du nom du fichier temporaire
   * unique a travers ce tube 
   */

  Envoyer_Message_Serveur (Chemin_Serveur, Nom_Article);

  /*
   * Maintenant, celui qui lance le posteur redevient l'utilisateur reel. Il
   * peut donc effacer le fichier contenant le corps du message dont il est
   * proprietaire
   */

  my_setreuid (ruid, ruid);

  if (remove (Nom_Corps_Article) != 0)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'effacement du fichier %s.\n",
	       __FILE__, __LINE__, Nom_Corps_Article);
      perror ("remove ");
      exit (errno);
    }

  return (0);

}
