/*
Copyright (C) 2000 Mathieu Decore <mdecor@ens.uvsq.fr>
                   Marc Tanguy <mtanguy@ens.uvsq.fr>

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le 
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence, ou
 encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE 
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER. 
Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
 temps que ce programme ; 
si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 
675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <sys/stat.h>
#include <signal.h>
#include <limits.h>
#include <ctype.h>
#include "serveur.h"

char Fichier_Lock[TAILLE_CHEMIN + TAILLE_NOM];
char *Chemin_Serveur;

int Vieux=0;
char Heure_MAJ=0;
char Minute_MAJ=0;

void Usage (void)
{
  printf ("Usage : serveur [-h] [-m heure] [-v jours] [-r num]\n");
  exit (0);
}

void Quitter_Serveur (int sig)
{ 
  unlink (Fichier_Lock);
  exit (0);  
}

int Lire_Fichier_Conf (char *buffer)
{
  FILE *Conf;
  char Fichier_Conf[TAILLE_NOM + TAILLE_CHEMIN];
  struct stat stbuf;

  strcpy (Fichier_Conf, Chemin_Serveur);
  strcat (Fichier_Conf, "/serveur.conf");

  if (stat (Fichier_Conf, &stbuf) != 0)
    {      
      /* Une autre erreur de stat que le fichier n'existe pas -> exit */

	if (errno != ENOENT)
	  { 
	    fprintf (stderr, 
		     "%s:%d : Erreur lors de l'appel de stat avec le fichier %s\n", 
		     __FILE__, __LINE__, Fichier_Conf);
	    fflush (stderr);
	    perror ("stat ");
	    exit (errno);
	  }
	else
	  {
	    return (0);
	  }
    }

  Conf = fopen (Fichier_Conf, "r");
  
  if (Conf == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de open avec le "
	       "fichier %s.\n", 
	       __FILE__, __LINE__, Fichier_Conf);
      fflush (stderr);
      
      perror ("open ");
      exit (errno);
    }
  
  fread (buffer, sizeof (char), TAILLE_BUF, Conf);

  buffer[strlen (buffer) - 1] = EOF;

  return (1);
}

int Lire_Fichier_MAJ ()
{
  char *buffer;

  buffer = (char *) malloc (TAILLE_BUF * sizeof (char));

  if (buffer == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur lors de malloc Chemin_Serveur\n",
	       __FILE__, __LINE__);
      fflush (stderr);
      Quitter_Serveur (0);
      exit (101);
    }

  memset (buffer, 0, TAILLE_BUF);

  if (!Lire_Fichier_Conf (buffer))
    {
      return (0);
    }

  while ((*buffer) != EOF)
    {
      if (!strncmp (buffer, "H_MAJ", strlen ("H_MAJ")))
	{	  
	  printf ("%s\n", strtok (buffer, " "));
	  fflush (stdout);
	  Heure_MAJ = atoi (strtok (NULL, ""));
	  printf ("Heure de mise a jour : %d h.\n",
		  Heure_MAJ);
	  fflush (stdout);
	}

      if (!strncmp (buffer, "MN_MAJ", strlen ("MN_MAJ")))
	{
	  printf ("%s\n", strtok (buffer, " "));
	  fflush (stdout);
	  Minute_MAJ = atoi (strtok (NULL, ""));
	  printf ("Minute de mise a jour : %d h.\n",
		  Minute_MAJ);
	  fflush (stdout);
	}

      /*
       * On avance le pointeur jusqu'a la fin de la ligne
       */

      while (((*buffer) != '\n') && ((*buffer) != EOF))
	{
	  (buffer)++;
	}

      /*
       * On deplace le pointeur sur le debut de la ligne suivante
       */

      if (*buffer == '\n')
	{
	  (buffer)++;      
	}
    }

  printf ("\n");
  fflush (stdout);

  return (1);
}

int Lire_Fichier_Vieux ()
{
  char *buffer;

  buffer = (char *) malloc (TAILLE_BUF * sizeof (char));

  if (buffer == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur lors de malloc Chemin_Serveur\n",
	       __FILE__, __LINE__);
      fflush (stderr);
      Quitter_Serveur (0);
      exit (101);
    }

  memset (buffer, 0, TAILLE_BUF);

  if (!Lire_Fichier_Conf (buffer))
    {
      return (0);
    }

  while ((*buffer) != EOF)
    {
      if (!strncmp (buffer, "VIEUX", strlen ("VIEUX")))
	{
	  printf ("%s\n", strtok (buffer, " "));
	  fflush (stdout);
	  Vieux = atoi (strtok (NULL, ""));
	  printf (
		  "Mise a jour : tous les articles plus vieux de %d jour(s) "
		  "seront effaces.\n",
		  Vieux);
	  fflush (stdout);
	}
      
      /*
       * On avance le pointeur jusqu'a la fin de la ligne
       */

      while (((*buffer) != '\n') && ((*buffer) != EOF))
	{
	  (buffer)++;
	}

      /*
       * On deplace le pointeur sur le debut de la ligne suivante
       */

      if (*buffer == '\n')
	{
	  (buffer)++;      
	}
    }

  printf ("\n");
  fflush (stdout);

  return (1);
}

void Traiter_HUP (int sig)
{
    Lire_Fichier_Vieux ();
    Lire_Fichier_MAJ ();
}

void Trouver_Premier_Dernier_Article (char *repertoire, 
				      NbArticles_t *nombre_article,
				      NbArticles_t *premier_article, 
				      NbArticles_t *dernier_article)
{
  DIR * dirp;
  struct dirent * courant;
  struct stat type;
  time_t t;

  time_t date_premier_article = 0;
  time_t date_dernier_article = LONG_MAX;

  char Nom_Article[TAILLE_NOM + TAILLE_CHEMIN];
  
  if ((dirp = opendir (repertoire)) == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur d'ouverture du repertoire %s\n", 
	       __FILE__, __LINE__, repertoire);
      fflush (stderr);
      perror ("opendir ");
      exit (errno);
    }

  printf ("Determination du plus ancien et du plus recent article...\n\n");
  fflush (stdout);
 
  while ((courant = readdir (dirp)) != NULL)
    {
      if ((strcmp (courant->d_name, ".") != 0) &&
	  (strcmp (courant->d_name, "..") != 0))
	{	
	  
	  if (strlen (repertoire) + strlen (courant->d_name) <
	      TAILLE_NOM + TAILLE_CHEMIN)	   
	    {
	      memset (Nom_Article, 0, TAILLE_NOM);	
	      strcpy (Nom_Article, repertoire);
	      strcat (Nom_Article, courant->d_name);
	    }
	  else 
	    {
	      fprintf (stderr, 
		       "%s:%d : Erreur :\n%s \n ou \n%s \n trop grand\n", 
		       __FILE__, __LINE__, repertoire, courant->d_name);
	      fflush (stderr);
	      Quitter_Serveur (0);
	      exit (100);
	    }
	  
	  if ((stat(Nom_Article, &type)) != 0)
	    {
	      perror("Erreur stat");
	      exit (errno);
	    }

	  time (&t);

	  if (t - type.st_mtime > date_premier_article)
	    {
	      date_premier_article = t - type.st_mtime;
	      *premier_article = atoi (courant->d_name);
	    }
	  
	  if (t - type.st_mtime < date_dernier_article)
	    {
	      date_dernier_article = t - type.st_mtime;
	      *dernier_article = atoi (courant->d_name);
	    }
	  
	  (*nombre_article)++;
	}
    }
  
  if (errno != 0)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de readdir %s\n",
	       __FILE__, __LINE__, repertoire);	
      perror ("readdir ");
      exit (errno);
    }

  printf ("Premier article : %li Dernier Article : %li\n", *premier_article,
	  *dernier_article);
  fflush (stdout);

  closedir(dirp);
}

void Verifier_Lock (char *Chemin_Serveur)
{
  struct stat stbuf;

  strcpy (Fichier_Lock, Chemin_Serveur);
  strcat (Fichier_Lock, "/.serveur_lock");

/* Une erreur avec stat s'est produite */

  if (stat (Fichier_Lock, &stbuf) != 0)
    {
      if (errno == ENOENT) /* Si le fichier lock n'existe pas -> on le cree */
	{	  
	  errno = 0;
	  
	  if (open (Fichier_Lock, O_WRONLY | O_CREAT | O_TRUNC) == -1)
	    {
	      fprintf (stderr, "%s:%d : Erreur lors de open %s\n", 
		       __FILE__, __LINE__, Fichier_Lock);
	      perror ("open ");
	      exit (errno);
	    }
	  
	  printf ("Fichier verrou cree : %s\n", Fichier_Lock);
	}

      else /* Une autre erreur de stat que le tube n'existe pas -> exit */
	{
	  fprintf (stderr, 
		   "%s:%d : Erreur lors de l'appel de stat avec le fichier %s\n", 
		   __FILE__, __LINE__, Fichier_Lock);
	  fflush (stderr);
	  perror ("stat ");
	  exit (errno);
	}
    }

  else /* stat s'est bien passe -> le fichier existe */
    {
      fprintf (stderr, "%s:%d : Le serveur doit deja etre lance.\n", 
	       __FILE__, __LINE__);
      Quitter_Serveur (0);
      exit (102);
    }
  
}


void Verifier_Repertoire_Articles (char *repertoire)
{
  DIR * dirp;
  struct dirent * courant;
  
  char Nom_Article[TAILLE_NOM + TAILLE_CHEMIN];
  
  if ((dirp = opendir (repertoire)) == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur d'ouverture du repertoire %s\n", 
	       __FILE__, __LINE__, repertoire);
      fflush (stderr);
      perror ("opendir ");
      exit (errno);
    }
 
  printf ("Nettoyage du repertoire contenant les articles...\n\n");
  fflush (stdout);
 
  while ((courant = readdir (dirp)) != NULL)
    {
      if ((strcmp (courant->d_name, ".") != 0) &&
	  (strcmp (courant->d_name, "..") != 0))
	{	
	  
	  if (strlen (repertoire) + strlen (courant->d_name) <
	      TAILLE_NOM + TAILLE_CHEMIN)	   
	    {
	      memset (Nom_Article, 0, TAILLE_NOM);	
	      strcpy (Nom_Article, repertoire);
	      strcat (Nom_Article, courant->d_name);
	    }
	  else 
	    {
	      fprintf (stderr, 
		       "%s:%d : Erreur :\n%s \n ou \n%s \n trop grand\n", 
		       __FILE__, __LINE__, repertoire, courant->d_name);
	      fflush (stderr);
	      Quitter_Serveur (0);
	      exit (100);
	    }
	  
	  printf ("Effacement de %s\n", Nom_Article);
	  fflush (stdout);
	  
	  if (unlink (Nom_Article) == -1)
	    {
	      fprintf (stderr, "%s:%d : Erreur lors de unlink %s\n", 
		       __FILE__, __LINE__, Nom_Article);
	      fflush (stderr);
	      perror ("unlink ");
	      closedir(dirp);
	      exit (errno);
	    }	  
	}
    }	

  if (errno != 0)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de readdir %s\n",
	       __FILE__, __LINE__, repertoire);	
      perror ("readdir ");
      exit (errno);
    }
  
  printf ("\nNettoyage fait.\n\n");
  fflush (stdout);
  closedir(dirp);
}

void Effacer_Anciens_Articles (time_t t, int Vieux, char *Chemin_Articles, 
			       NbArticles_t *nombre_article,
			       NbArticles_t *premier_article,
			       NbArticles_t *dernier_article)
{
  DIR * dirp;
  struct dirent * courant;
  struct stat type;
  
  char Nom_Article[TAILLE_NOM + TAILLE_CHEMIN];
    
  if ((dirp = opendir (Chemin_Articles)) == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur d'ouverture du repertoire\n", 
	       __FILE__, __LINE__);
      fflush (stderr);
      perror ("opendir ");
      exit (errno);
    }
  
  while ((courant = readdir(dirp)) != NULL)
    {
      
      if ((strcmp (courant->d_name, ".") != 0) &&
	  (strcmp (courant->d_name, "..") != 0))
	{
	  if (strlen (Chemin_Articles) + strlen (courant->d_name) <
	      TAILLE_NOM + TAILLE_CHEMIN)
	    {
	      memset (Nom_Article, 0, TAILLE_NOM);
	      strcpy (Nom_Article, Chemin_Articles);
	      strcat (Nom_Article, courant->d_name);
	    }
	  
	  if ((stat(Nom_Article, &type)) != 0)
	    {
	      perror("Erreur stat");
	      exit (errno);
	    }
	  
	  if ((t - type.st_mtime) > (Vieux*86400))
	    {
	      printf ("Effacement de %s\n", Nom_Article);
	      fflush (stdout);
	      
	      if (unlink (Nom_Article) == -1)
		{
		  fprintf (stderr, "%s:%d : Erreur lors de unlink %s\n", 
			   __FILE__, __LINE__, Nom_Article);
		  perror ("unlink ");
		  exit (errno);
		}
	      (*nombre_article)--;
	      (*premier_article)++;

	      /*
	       * Si on efface *tous* les articles, il faut remettre a jour 
	       * les valeurs de premier_article et dernier_article
	       */

	      if (*premier_article > *dernier_article)
		{
		  *dernier_article = *premier_article-1;
		}
	      printf ("nombre_article=%li premier_article=%li\n", 
		      (*nombre_article), (*premier_article));
	    }
	}
      printf ("Premier article : %li Dernier Article : %li\n", *premier_article,
	      *dernier_article);
      fflush (stdout);

    }
  
  if (errno != 0)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de readdir %s\n",
	       __FILE__, __LINE__, Chemin_Articles);
      perror ("readdir ");
      exit (errno);
    } 
}


void MAJ_Fichier_Forum (char *Fichier_Forum, FILE *Forum,
			NbArticles_t nombre_article,
			NbArticles_t premier_article, 
			NbArticles_t dernier_article)
{
  
  char buffer [TAILLE_BUF];

  truncate (Fichier_Forum, 0);
  rewind (Forum);

  sprintf (buffer, "%li %li %li", nombre_article, premier_article, 
	   dernier_article);
  
  fwrite (buffer, sizeof (char), strlen (buffer), Forum);

  
  if (fflush (Forum) != 0)
    {
      fprintf (stderr, 
	       "%s:%d : Erreur lors de fflush %s.\n",
	       __FILE__, __LINE__, Fichier_Forum);
      
      perror ("fflush ");
      exit (errno);
    }
  
}


void Mise_a_jour (time_t t, int Vieux, char *Chemin_Articles, 
		  NbArticles_t *nombre_article,
		  NbArticles_t *premier_article,
		  NbArticles_t *dernier_article)
{
  
  printf ("MISE A JOUR\n\n");  
  fflush (stdout);
  Effacer_Anciens_Articles (t, Vieux, Chemin_Articles, nombre_article, 
			    premier_article, dernier_article);
  printf ("FIN MISE A JOUR\n\n");  
}

int Tube_Existe (char *Nom_Tube, struct stat *stbuf)
{

  if (stat (Nom_Tube, stbuf) != 0) /* Une erreur avec stat s'est produite */
    {
      if (errno == ENOENT) /* Si le tube n'existe pas -> retourne 0 */
	{
	  errno = 0;
	  
	  return (0);
	}
      else /* Une autre erreur de stat que le tube n'existe pas -> exit */
	{
	  fprintf (stderr, 
		   "%s:%d : Erreur lors de l'appel de stat avec le fichier"
		   "%s.\n", 
		   __FILE__, __LINE__, Nom_Tube);
	  fflush (stderr);
	  perror ("stat ");
	  exit (errno);
	}
    }
  
  return (1); /* Aucune erreur avec stbuf : le tube existe deja -> 
		 retourne 1 */

}

void Verifier_Droits (char *Nom_Tube, struct stat stbuf)
{
  if (stbuf.st_mode != 4480) /* Si le tube n'a pas les bons droits */
    { 
      if (chmod (Nom_Tube, 4480) != 0) /* On met a jour les droits */
	{
	  fprintf (stderr, 
		   "%s:%d : Erreur lors de l'appel de chmod avec le fichier"
		   "%s.\n", 
		   __FILE__, __LINE__, Nom_Tube);
	  fflush (stderr);

	  perror ("chmod ");
	  exit (errno);
	}
      printf ("Droits du tube mis a jour (4480).\n");      
      fflush (stdout);
    }
}

int Ouvrir_Tube (char *Nom_Tube)
{

  int Tube_FD;

  Tube_FD = open (Nom_Tube, O_RDONLY | O_NDELAY);
  
  if (Tube_FD == -1)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de open avec le "
	       "fichier %s.\n", 
	       __FILE__, __LINE__, Nom_Tube);
      fflush (stderr);

      perror ("open ");
      exit (errno);
    }

  return (Tube_FD);

}

int Creer_Tube_Nomme (char *Chemin_Serveur)
{

  char Nom_Tube [TAILLE_NOM + TAILLE_CHEMIN];  
  struct stat stbuf;

  /*  printf ("Chemin_Serveur : %s\n", Chemin_Serveur); */
  

  strcpy (Nom_Tube, Chemin_Serveur);
  strcat (Nom_Tube, "/newspipe");  


  /* 
   * Si le tube n'existe pas deja, on le cree 
   */

  if (!Tube_Existe (Nom_Tube, &stbuf))
    {
	if (mkfifo (Nom_Tube, 0600) != 0)
	  { 
	    fprintf (stderr, 
		     "%s:%d : Erreur lors de l'appel de mkfifo avec le "
		     "fichier %s.\n", 
		     __FILE__, __LINE__, Nom_Tube);
	    fflush (stderr);

	    perror ("mkfifo ");
	    exit (errno);
	  }
	else /* Le tube a ete cree */
	  {
	    printf ("Tube nomme %s cree.\n\n", Nom_Tube);
	    fflush (stdout);
	  }
    }

  else /* Le tube existe deja -> pas de nouvelle creation */
    {
      printf ("Le tube \n\t %s\nexiste deja, pas de nouvelle creation.\n\n", 
	      Nom_Tube);
      fflush (stdout);
    }

  /* 
   * Verification des droits du tube et mise a jour si besoin
   */

  Verifier_Droits (Nom_Tube, stbuf);

  /* 
   * Retourne le descripteur de fichier associe au tube nomme
   */

  return (Ouvrir_Tube (Nom_Tube));

}

FILE * Creer_Fichier_Forum (char *Fichier_Forum)
{
  FILE *Forum;
  
  Forum = fopen (Fichier_Forum, "w+");
  
  if (Forum == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur lors de l'appel de open avec le "
	       "fichier %s.\n", 
	       __FILE__, __LINE__, Fichier_Forum);
      fflush (stderr);
      
      perror ("open ");
      exit (errno);
    }

  chmod (Fichier_Forum, 0644);

  return (Forum);
}


int main (int argc, char *argv[])
{
  int Tube_FD;  
  FILE *Forum;
  
  char Fichier_Forum[TAILLE_NOM + TAILLE_CHEMIN];

  NbArticles_t nombre_article = 0;
  NbArticles_t premier_article = 0;
  NbArticles_t dernier_article = -1;
  
  char nom_local_article[TAILLE_CHEMIN + TAILLE_NOM];
  /*  char buffer[TAILLE_BUF]; */
  
  char Chemin_Articles[TAILLE_CHEMIN];

  char nom_article[TAILLE_CHEMIN + TAILLE_NOM];

  char lu[TAILLE_CHEMIN + TAILLE_NOM];
 
  char flag;
  char Maj_def = 0;
  char Vieux_def = 0;

  struct tm *tm; 
  time_t t;

  int i;
  
  int Menage = 1;
  flag = 1;

  if (signal(SIGINT, Quitter_Serveur) == SIG_ERR)
    {
      perror("signal");
      exit(errno);
    }

  if (signal(SIGQUIT, Quitter_Serveur) == SIG_ERR)
    {
      perror("signal");
      exit(errno);
    }

  if (signal(SIGABRT, Quitter_Serveur) == SIG_ERR)
    {
      perror("signal");
      exit(errno);
    }

  if (signal(SIGTERM, Quitter_Serveur) == SIG_ERR)
    {
      perror("signal");
      exit(errno);
    }

  if (signal(SIGHUP, Traiter_HUP) == SIG_ERR)
    {
      perror("signal");
      exit(errno);
    }
  

  /*
   * Allocation memoire
   */
  
  Chemin_Serveur = (char *) malloc (TAILLE_CHEMIN * sizeof (char));

  if (Chemin_Serveur == NULL)
    {
      fprintf (stderr, "%s:%d : Erreur lors de malloc Chemin_Serveur\n",
	       __FILE__, __LINE__);
      Quitter_Serveur (0);
      exit (101);
    }

  /* 
   * Creation du tube nomme et ouverture en lecture seule
   */

  for (i=1; i<argc; i++)
    { 
      if ((!strcmp (argv[i], "-h")) || (!strcmp (argv[i], "--help")))
	{
	  Usage ();
	}

      else if ((!strcmp (argv[i], "-m")) || (!strcmp (argv[i], "--maj")))
	{
	  Heure_MAJ = atoi (strtok (argv[i+1], ":"));
	  Minute_MAJ = atoi (strtok (NULL, ""));
	  
	  Maj_def = 1;
	}

      else if ((!strcmp (argv[i], "-v")) || (!strcmp (argv[i], "--vieux")))
	{
	  Vieux = atoi (argv[i+1]);
	  Vieux_def = 1;
	}

      else if ((!strcmp (argv[i], "-r")) || (!strcmp (argv[i], "--relancer")))
	{
	  Menage = 0;
	}
    }

  strcpy (Chemin_Serveur, CHEMIN_SERVEUR);
  printf ("Chemin du serveur utilise : %s\n", Chemin_Serveur);

  strcpy (Chemin_Articles, Chemin_Serveur);
  strcat (Chemin_Articles, "/Articles/");

  printf ("Chemin des articles utilise : %s\n", Chemin_Articles);

  Verifier_Lock (Chemin_Serveur);

  strcpy (Fichier_Forum, Chemin_Serveur);
  strcat (Fichier_Forum, "/.forumSRV");

  Forum = Creer_Fichier_Forum (Fichier_Forum);

  if (Menage)
    {
      Verifier_Repertoire_Articles (Chemin_Articles);
    }

  else
    {
      Trouver_Premier_Dernier_Article (Chemin_Articles, &nombre_article,
				       &premier_article, &dernier_article);
      MAJ_Fichier_Forum (Fichier_Forum, Forum, nombre_article, 
			 premier_article, dernier_article);
    }

  if (!Vieux_def)
    {
      if (Lire_Fichier_Vieux ())
	{
	  printf ("Prise en compte du fichier de configuration.\n");
	}

      else
	{
	  Vieux = VIEUX;
	}
    }

  if (Vieux < 0)
    {
      fprintf (stderr, "%s:%d : Erreur : Vieux negatif (%d).\n",
	       __FILE__, __LINE__, Vieux);
      Quitter_Serveur (0);
      exit (105);
    }

  printf ("Les articles de plus de %d jour(s) seront effaces.\n", Vieux);

  if (!Maj_def)
    {      
      if (Lire_Fichier_MAJ ())
	{
	  printf ("Prise en compte du fichier de configuration.\n");
	}

      else
	{
	  Heure_MAJ = H_MAJ;
	  Minute_MAJ = MN_MAJ;
	}
    }

  if ((Heure_MAJ >= 24) || (Minute_MAJ >= 60))
    {
      fprintf (stderr, "%s:%d : Heure non valide (%d h %d).\n\n",
	       __FILE__, __LINE__, Heure_MAJ, Minute_MAJ);
      Quitter_Serveur (0);
      exit (106);

    }

  printf ("Heure de mise a jour : ");
  printf ("%d h %d mn.\n", Heure_MAJ, Minute_MAJ);

  MAJ_Fichier_Forum (Fichier_Forum, Forum, nombre_article, premier_article,
		     dernier_article+1);

  Tube_FD = Creer_Tube_Nomme (Chemin_Serveur);

  /*
   * Boucle
   */

  while (1)
    {
      time (&t);
      tm = localtime (&t);
      
      if ((tm->tm_hour == Heure_MAJ) && (tm->tm_min == Minute_MAJ))
	{
	  if (flag)
	    {
	      Mise_a_jour (t, Vieux, Chemin_Articles, &nombre_article, 
			   &premier_article, &dernier_article);
	      MAJ_Fichier_Forum (Fichier_Forum, Forum, nombre_article, 
				 premier_article, dernier_article);
	      flag = 0;
	    }
	}

      else
	{
	  flag = 1;
	}

      memset (lu, 0, TAILLE_NOM) ;
      memset (nom_local_article, 0, TAILLE_NOM_LOCAL_ARTICLE);
      memset (nom_article, 0, TAILLE_NOM_LOCAL_ARTICLE);

      /* 
       * Lecture d'un nom de fichier temporaire contenant l'article
       */
      
      if (read (Tube_FD, lu, 
	    TAILLE_CHEMIN + TAILLE_NOM) > 0)
	{
	  /*	  printf ("Nom du fichier lu : %s\n", lu);  */

	  /*
	   * Attribution du nom de l'article pour le serveur
	   */

	  sprintf (nom_local_article, "%li", dernier_article+1);

	  /*	  printf ("Nom du fichier local : %s\n", nom_local_article); */

	  /*
	   * Debut du nom de l'article : son repertoire, contenu dans
	   * CHEMIN_ARTICLE definit a la compilation
	   */
	  
	  strcpy (nom_article, Chemin_Articles);

	  /*	  printf ("Nom du chemin : %s\n", nom_article); */

	  /* 
	   * On ajoute le nom de l'article, contenu dans nom_local_article
	   * (le nom de l'article est un nombre). Au final, le nom de
	   * l'article nom_article est de la forme :
	   * CHEMIN_ARTICLE/nom_local_article
	   */

	  if (strlen (nom_article) + strlen (nom_local_article) <
	      TAILLE_NOM + TAILLE_CHEMIN)
	    {
	      if (strcat (nom_article, nom_local_article) == NULL)
		{		
		  fprintf (stderr, "%s:%d : Erreur lors de l'appel de strcat "
			   "%s %s\n",
			   __FILE__, __LINE__, nom_article, nom_local_article);
		  perror ("strcat ");
		  exit (errno);
		}
	    }
	  
	  else
	    {
	      fprintf (stderr, "%s:%d : Erreur : \n%s \n ou \n%s \n trop "
		       "grand\n", 
		       __FILE__, __LINE__, nom_article, nom_local_article);
	      Quitter_Serveur (0);
	      exit (100);
	    }

	      /*	  printf ("nom_article %s\n", nom_article); */
	  
	  /*
	   * Le nom de l'article suivant aura le nom de l'article actuel plus
	   * 1
	   */

	  nombre_article++;
	  dernier_article++;
	  
	  /*
	   * Renommage du fichier temporaire en fichier local au serveur
	   * et attribution des droits sur ce fichier
	   */

	  printf ("renommage %s -> %s\n\n", lu, nom_article);
	  fflush (stdout);

	  if (rename (lu, nom_article) == -1)
	    {
		fprintf (stderr, "%s:%d : Erreur lors du renomage de\n%s "
			 "en\n%s\n",
			__FILE__, __LINE__, lu, nom_article);
		fflush (stderr);
	      perror ("rename ");
	      exit (errno);
	    }

	  MAJ_Fichier_Forum (Fichier_Forum, Forum, nombre_article, 
			     premier_article, dernier_article);
	  
	}

      else
	{
	  sleep (1);
	  errno = 0;
	}
    }

  close (Tube_FD);  
  exit (0); 
}
